local name, addon = ...
local parentName = addon['parentName'] or "BrokerPack"
local childName = "zz_Coords"
-- Provides a basic interface for something that manages the adding, updating, and removing of data like icons, blobs or text to the map canvas
local zzDataProvider = {};
local coordText,db
local mapframe
local zzDebug = false

function zzDataProvider:GetMap()
	return self.owningMap;
end
function addon:GetXY()
	local x, y = WorldMapFrame.ScrollContainer:GetNormalizedCursorPosition();
	local adjustedX = x * 100
	local adjustedY = y * 100
	return adjustedX,adjustedY
end
local function update2()
	if(not addon['db']['profile'][childName]) then return end
	if(not db) then
		db = addon['db']['profile'][childName]
	end
	if(coordText) then
		addon:setOldPlayerCoords()
		local x, y = GetCursorPosition();
		x = x / WorldMapButton:GetEffectiveScale();
		y = y / WorldMapButton:GetEffectiveScale();

		local centerX, centerY = WorldMapButton:GetCenter();
		local width = WorldMapButton:GetWidth();
		local height = WorldMapButton:GetHeight();
		local adjustedY = addon:round((centerY + (height/2) - y) / height * 100,1);
		local adjustedX = addon:round((x - (centerX - (width/2))) / width * 100,1);
		coordText:SetText(format("Mouse: %i, %i Player: %s",adjustedX,adjustedY,addon:GetOldPlayerCoords()))
		if(not db['hideMapCoords']) then
			mapframe:SetWidth(coordText:GetStringWidth()+4)
			mapframe:SetPoint("BOTTOMLEFT", WorldMapButton, "BOTTOMLEFT", db['windowX'], db['windowY'])
			mapframe:Show()
		else
			mapframe:Hide()
		end
	end
end
local function update()
	if(not addon['db']['profile'][childName]) then return end
	if(not db) then
		db = addon['db']['profile'][childName]
	end
	if(coordText) then
		db['screenX'] = db['screenX'] or 10
		db['screenY'] = db['screenY'] or 5
		db['windowX'] = db['windowX'] or 10
		db['windowY'] = db['windowY'] or 5
		local x,y = addon:GetXY()
		local str = ""
		local mapID = zzDataProvider:GetMap():GetMapID()
		if(mapID and db['ShowMapID']) then
			str = format("%s MapID: %i",str,mapID)
		end
		if(not(x == 100 or y==100 or x==0 or y==0)) then
			str = format("%s Mouse: %.1f / %.1f",str,x,y)
		end
		if(mapID) then
			local x,y = C_Map.GetPlayerMapPosition(mapID,"player"):GetXY()
			if(x and y) then
				str = format("%s Player: %.1f / %.1f",str,x * 100,y * 100)
			end
		end
		coordText:SetText(str)
		if(not db['hideMapCoords']) then
			mapframe:SetWidth(coordText:GetStringWidth()+4)
			mapframe:SetPoint("BOTTOMLEFT", WorldMapFrame, "BOTTOMLEFT", db['windowX'], db['windowY'])
			mapframe:Show()
		else
			mapframe:Hide()
		end

	end
end
function zzDataProvider:OnAdded(owningMap)
	-- Optionally override in your mixin, called when this provider is added to a map canvas
	self.owningMap = owningMap;
	if(not coordText or not mapframe) then
		mapframe = CreateFrame("Frame", "zz_CoordsMapFrame", C_Map and WorldMapFrame or WorldMapButton)
		mapframe:SetWidth(600)
		mapframe:SetHeight(20)
--		mapframe:SetPoint("BOTTOMLEFT", WorldMapFrame, "BOTTOMLEFT", addon['db']['windowX'], addon['db']['windowY'])
		mapframe:SetFrameStrata("HIGH")
		mapframe:SetBackdrop({	
			bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		})
		mapframe:SetBackdropColor(0,0,0,1);
		mapframe.elapsed = 0
		mapframe:SetScript("OnEnter", function(self)
			self:SetBackdropColor(0,0,0,0);
			coordText:Hide()
		end)
		mapframe:SetScript("OnLeave", function(self)
			coordText:Show()
			self:SetBackdropColor(0,0,0,1);
		end)
		mapframe:EnableMouse(1)
		coordText = mapframe:CreateFontString(nil, "OVERLAY","GameFontNormal")
		coordText:SetAllPoints(mapframe)
		coordText:SetJustifyH("LEFT")
		coordText:SetShadowColor(1,1,1,.2)
		coordText:SetShadowOffset(1,-1)
		mapframe:SetScript("OnUpdate", function(self, elapsed)
			self.elapsed = self.elapsed + elapsed
			if(self.elapsed<0.2) then return end
			self.elapsed = 0
			if(C_Map) then
				update()
			else
				update2()	
			end
		end)
		addon['mapframe'] = mapframe
	end
end

function zzDataProvider:OnRemoved(owningMap)
	-- Optionally override in your mixin, called when this provider is removed from a map canvas
	assert(owningMap == self.owningMap);
	self.owningMap = nil;

	if self.registeredEvents then
		for event in pairs(self.registeredEvents) do
			owningMap:UnregisterEvent(event);
		end
		self.registeredEvents = nil;
	end
end

function zzDataProvider:RemoveAllData()
	-- Override in your mixin, this method should remove everything that has been added to the map
	if(mapframe) then
		mapframe:Hide()
		mapframe = nil
		coordText = nil
	end
end

function zzDataProvider:RefreshAllData(fromOnShow)
	-- Override in your mixin, this method should assume the map is completely blank, and refresh any data necessary on the map
	update()
end

function zzDataProvider:OnShow()
	-- Override in your mixin, called when the map canvas is shown
end

function zzDataProvider:OnHide()
	-- Override in your mixin, called when the map canvas is closed
end

function zzDataProvider:OnMapInsetSizeChanged(mapInsetIndex, expanded)
	-- Optionally override in your mixin, called when a map inset changes sizes
end

function zzDataProvider:OnMapInsetMouseEnter(mapInsetIndex)
	-- Optionally override in your mixin, called when a map inset gains mouse focus
end

function zzDataProvider:OnMapInsetMouseLeave(mapInsetIndex)
	-- Optionally override in your mixin, called when a map inset loses mouse focus
end

function zzDataProvider:OnCanvasScaleChanged()
	-- Optionally override in your mixin, called when the canvas scale changes
end

function zzDataProvider:OnCanvasPanChanged()
	-- Optionally override in your mixin, called when the pan location changes
end

function zzDataProvider:OnCanvasSizeChanged()
	-- Optionally override in your mixin, called when the canvas size changes
end

function zzDataProvider:OnEvent(event, ...)
	-- Override in your mixin to accept events register via RegisterEvent
end

function zzDataProvider:OnGlobalAlphaChanged()
	-- Optionally override in your mixin if your data provider obeys global alpha, called when the global alpha changes
end

function zzDataProvider:OnMapChanged()
	--  Optionally override in your mixin, called when map ID changes
	self:RefreshAllData();
end

function zzDataProvider:RegisterEvent(event)
	-- Since data providers aren't frames this provides a similar method of event registration, but always calls self:OnEvent(event, ...)
	if not self.registeredEvents then
		self.registeredEvents = {}
	end
	if not self.registeredEvents[event] then
		self.registeredEvents[event] = true;
		self:GetMap():AddDataProviderEvent(event);
	end
end

function zzDataProvider:UnregisterEvent(event)
	if self.registeredEvents and self.registeredEvents[event] then
		self.registeredEvents[event] = nil;
		self:GetMap():RemoveDataProviderEvent(event);
	end
end

function zzDataProvider:SignalEvent(event, ...)
	if self.registeredEvents and self.registeredEvents[event] then
		self:OnEvent(event, ...);
	end
end
if(type(WorldMapFrame.AddDataProvider) == 'function') then
	WorldMapFrame:AddDataProvider(zzDataProvider)
else
	zzDataProvider:OnAdded()
end